@@ -19,6 +19,8 @@
 // ***************************************************************************
 // |docname| - Core code for the CodeChat Client
 // ***************************************************************************
+"use strict";
+
 // Constants
 // =========
 // .. _GetResultType JS:
@@ -32,7 +34,7 @@ const GetResultType = {
 };
 
 // A regex to match a percentage: one or more digits, optionally followed by a period and one or more digits, and ending with a percent sign.
-let percent_regex = new RegExp(
+const percent_regex = new RegExp(
     // Capture the number in front of the percent sign.
     "(" +
         // Look for one or more digits, ...
@@ -57,13 +59,13 @@ function run_client(
     // Set up variables used by the functions below
     // --------------------------------------------
     // Get commonly-used nodes in the DOM.
-    let status_message = document.getElementById("status_message");
-    let build_progress = document.getElementById("build_progress");
-    let status_errors_div = document.getElementById("status_errors");
-    let outputElement = document.getElementById("output");
-    let build_div = document.getElementById("build");
-    let build_contents = document.getElementById("build_contents");
-    let errors_div = document.getElementById("errors");
+    const status_message = document.getElementById("status_message");
+    const build_progress = document.getElementById("build_progress");
+    const status_errors_div = document.getElementById("status_errors");
+    const outputElement = document.getElementById("output");
+    const build_div = document.getElementById("build");
+    const build_contents = document.getElementById("build_contents");
+    const errors_div = document.getElementById("errors");
 
     // True if the output/errors should be cleared on the next result.
     let clear_output = true;
@@ -83,9 +85,40 @@ function run_client(
     set_splitter_percent(splitter_size[errors_or_warnings]);
 
     // Create a websocket to communicate with the CodeChat Server.
-    let ws = new ReconnectingWebSocket(
-        `ws://${window.location.hostname}:${ws_port}`
-    );
+    //
+    // If the hosting page uses HTTPS, then use a secure websocket (WSS protocol); otherwise, use an insecure websocket (WS).
+    const protocol = window.location.protocol === "http:" ? "ws" : "wss";
+    // Compute the URI for this websocket, dealing with special cases for CoCalc and GitHub Codespaces.
+    let ws_uri;
+    const is_cocalc = window.location.hostname === "cocalc.com";
+    const is_gihub_codespaces = window.location.hostname.endsWith("github.dev");
+    if (is_cocalc || !is_gihub_codespaces) {
+        // A special case for CoCalc: use a different URL per the `CoCalc docs <https://doc.cocalc.com/howto/webserver.html>`_.
+        const separator = is_cocalc ? "/" : ":";
+        // The pathname is all but the last one or two elements of the hosting page's pathname: transform ``/a/long/path/to/the/client`` to ``/a/long/path/to/the``. For CoCalc, this transforms ``/f1d3f8ac-39da-48fe-9357-7d5c4ee132de/server/27377/client`` into ``/f1d3f8ac-39da-48fe-9357-7d5c4ee132de/server``.
+        const pathname = window.location.pathname
+            .split("/")
+            .slice(0, is_cocalc ? -2 : -1)
+            .join("/");
+        // Transform the hosting page's URL for the websocket. For example, transform from ``http://foo.org/client?id=0`` into ``ws://foo.org:27377``.
+        ws_uri = `${protocol}://${window.location.hostname}${pathname}${separator}${ws_port}`;
+    } else {
+        // We're using GitHub CodeSpaces, where the ``window.location`` is (for example) ``https://bjones1-organic-goggles-wv46gpj9x4cv5qq-27377.preview.app.github.dev/client?id=0``. The desired URI for this example is ``wss://bjones1-organic-goggles-wv46gpj9x4cv5qq-27378.preview.app.github.dev``.
+        //
+        // Throw away everything after the last hyphen.
+        const hostname_prefix = window.location.hostname
+            .split("-")
+            .slice(0, -1)
+            .join("-");
+        // Throw away everything before the first period.
+        const hostname_suffix = window.location.hostname
+            .split(".")
+            .slice(1)
+            .join(".");
+        // Now, build the desired URI.
+        ws_uri = `${protocol}://${hostname_prefix}-${ws_port}.${hostname_suffix}`;
+    }
+    const ws = new ReconnectingWebSocket(ws_uri);
 
     // Identify this client on connection.
     ws.onopen = () => {
@@ -108,13 +141,13 @@ function run_client(
 
     // Handle messages.
     ws.onmessage = (event) => {
-        result = JSON.parse(event.data);
+        const result = JSON.parse(event.data);
         if (result.get_result_type === GetResultType.url) {
             console.log(
                 `CodeChat Client: URL ${result.text} received; loading...`
             );
             // Save and restore scroll location through the content update, if we can.
-            let [scrollX, scrollY] = getScroll();
+            const [scrollX, scrollY] = getScroll();
             // See ideas in https://stackoverflow.com/a/16822995. Works for same-domain only.
             outputElement.onload = function () {
                 if (is_user_navigation) {
@@ -156,7 +189,8 @@ function run_client(
                 window.location.pathname +
                 "/" +
                 id +
-                "/" +
+                // Windows pathnames don't begin with a slash; Linux/Mac pathnames do. Avoid repeating the slash on Linux/Mac to get prettier URLs.
+                (result.text[0] === "/" ? "" : "/") +
                 result.text;
             // The next build output received will apply to the new build, so set the flag.
             clear_output = true;
@@ -186,7 +220,7 @@ function run_client(
                 build_div.textContent += result.text;
             }
             // Look for a percentage, to update the progress bar.
-            let percent_matches = result.text.match(percent_regex);
+            const percent_matches = result.text.match(percent_regex);
             if (percent_matches !== null) {
                 // Update the progress bar with the percentage from the last match.
                 build_progress.value = parseFloat(
@@ -305,7 +339,7 @@ function run_client(
 }
 
 // Globally-accessible functions (see above). These must be declared as ``var`` (not ``let``) due to (I assume) scoping rules I don't understand.
-var navigate_to_error, save_file;
+var navigate_to_error, save_file, browser_navigation;
 
 // Utilities
 // =========
@@ -316,7 +350,7 @@ function get_splitter() {
 
 // Set the splitter percentage
 function set_splitter_percent(percent) {
-    let splitter = get_splitter();
+    const splitter = get_splitter();
     splitter.percent = percent;
     splitMe.update(splitter);
 }
@@ -394,7 +428,7 @@ function parse_for_errors(errors_html) {
             }
 
             // Unsanitize the file name.
-            let div = document.createElement("div");
+            const div = document.createElement("div");
             div.innerHTML = file_path;
             file_path = div.textContent;
             // Make an unspecified filename empty.
